// ========================================================
//
// CBP_2403_D_Block_NonBlock  
// Looking at blocking and unblocking move forward
// Dead reckoning with experimental correction
// Works well
// CBP 10-09-21 
// Status Devel
//
// ========================================================

#include <stdio.h>
#include <Servo.h>

// Define pins ---------------
#define HC_SR04_Tx 2
#define HC_SR04_Rx 3
#define ServoL_pin 13
#define ServoR_pin 12

// Define constants ----------
#define dT 20
#define DEBUG true
#define wheelDiam 66.0
#define wheelRad (wheelDiam/2.0)
#define axleLen 104.0

// Global variables ----------
Servo servoL;
Servo servoR; 
float driveL, driveR;
float dist;

void setup() {
  Serial.begin(9600);
  cprintf(">>> Initializing stuff ..................\n");
  pinMode(HC_SR04_Tx,OUTPUT);
  pinMode(HC_SR04_Rx,INPUT);
  servoL.attach(ServoL_pin);
  servoR.attach(ServoR_pin);
  // Stop any servo movement
  driveServos(0.0,0.0);  
  
  cprintf(">>> Pausing 2 secs ......................\n");
  delay(2000);
  cprintf(">>> Starting  ...........................\n");

}


void loop() {
  moveForward_NonBlock(100);
  delay(1000);
  // 20 ms delay needed by SR_HC04
  delay(20);
}

float getDistance() {
  long duration;
  float mm;
  digitalWrite(HC_SR04_Tx, LOW);
  delayMicroseconds(2);
  digitalWrite(HC_SR04_Tx, HIGH);
  delayMicroseconds(10);
  digitalWrite(HC_SR04_Tx, LOW);

  duration = pulseIn(HC_SR04_Rx, HIGH);
  mm = 10*duration / 29 / 2;
  //if(DEBUG) cprintf("Dist (mm) = %f\n",mm);
  return mm;
  
}

void moveForward_Block(float mm) {
  float driveTime;
  float omega;
  float rpm;  
  float vely;
  
  driveL = 40;
  driveR = 40;

  rpm = (30.0/50.0)*driveL;
  omega = (2*PI/60.0)*rpm;
  vely = omega*wheelRad;
  driveTime = mm / vely;
  driveTime = driveTime*10.0/17.00; // Expt correction factor
  driveServos(driveL,driveR);
  
  if(DEBUG)
    cprintf("vely = %f (mm/s), omega = %f (rad/s) rpm = %f driveTime (ms) = %f\n",vely,omega,rpm,driveTime);

  delay((int)driveTime);
  
  driveL = 0;
  driveR = 0;
  driveServos(driveL,driveR);  
}

void moveForward_NonBlock(float mm) {
  float driveTime;
  float time;
  float dist;
  float omega;
  float rpm;  
  float vely;
    
  driveL = 25;
  driveR = 25;
  
  rpm = (30.0/50.0)*driveL;
  omega = (2*PI*rpm)/60.0;
  vely = omega*wheelRad;
  driveTime = mm / vely;
  driveTime = driveTime * 1000;
  
  driveTime = driveTime*10.0/17.00; // Expt correction factor
    
  driveServos(driveL,driveR); 
   
  if(DEBUG)
    cprintf("vely = %f (mm/s), omega = %f (rad/s) rpm = %f driveTime = %f\n",vely,omega,rpm,driveTime);
  
  time = millis();
  while(millis() - time < driveTime) {
   // time += dT;
    //time += 20.0;
    //cprintf("--------- Time = %f\n",time);
    dist = getDistance();
    delay(20); // needed by CH_SR04
    //cprintf("Dist = %f\n",dist);
    if(dist < 300 ) {
      if(DEBUG) cprintf("--> Break, dist is %f\n",dist);  
      break;
    }
  }

  driveL = 0;
  driveR = 0;
  driveServos(driveL,driveR);

}

void driveServos(float _driveL, float _driveR) {
  servoL.writeMicroseconds(1500 - (int)_driveL);
  servoR.writeMicroseconds(1500 + (int)_driveR);
}
