// ======================================================
// CBP_2403_R_MoveToBlock
// 23-10-21
// Rotates to object in FOV and tracks
// Uses LUT to convert width to distance
// FBO_CBP_MoveToBlock.ino
// FB0 CBP 06-07-21
// ======================================================

#include <Pixy2.h>
#include <Servo.h>

// Define pins ---------------
#define ServoL_pin 13
#define ServoR_pin 12
#define ServoTurret_pin 11

// Define constants ----------
#define DEBUG true
#define wheelDiam 66.0
#define wheelRad (wheelDiam/2.0)
#define axleLen 104.0

// Global variables ----------
Servo servoL;
Servo servoR; 
Servo servoTurret;

float driveL, driveR;

// This is the main Pixy object 
Pixy2 pixy;

// Pixy variables
uint16_t width;
uint16_t height;
uint16_t color;
uint8_t index;
uint16_t x;    // Must be 16 else caps at 255
uint16_t y;

int offset;

// controller variables
float error;
float Kp;
float Kd;
float lastError = 0;

uint16_t frameWidth;

void setup() {
  Serial.begin(9600);
  cprintf(">>> Initializing stuff ..................\n");
  Serial.begin(9600);
  Serial.print("Starting...\n");

  // Make sure camera is pointing forwards.  
  // Apply your offset here
  offset = 0;
  servoTurret.attach(ServoTurret_pin);
  // Now set it to 90 degs pointing forwards
  servoTurret.write(90 + offset);
  delay(1000);
  servoTurret.detach();

  // Init the camera
  pixy.init();
  // get details of the camera
  frameWidth = pixy.frameWidth;

  // Attach the servos
  servoL.attach(ServoL_pin);
  servoR.attach(ServoR_pin);
  cprintf(">>> Pausing 2 secs ......................\n");
  delay(2000);
  cprintf(">>> Starting  ...........................\n"); 
  cprintf("frameWidth = %d\n",frameWidth);

  // Proportional gain for controller
  Kp = 50.0;
  Kd = 0;
}

void loop() {
  float dist;
  float deriv;
  
  // Following line will return just one block, the largest
  pixy.ccc.getBlocks(true, 1, 1);

  // If we have a block then print out its width
  if (pixy.ccc.numBlocks > 0) {
    x = pixy.ccc.blocks[0].m_x;
    // calc the error
    
    // normalize the error to range -1.0 to 1.0
    
    // use the error to create the drives
    //driveR = ;
    //driveL = ;
    driveServos(driveL,driveR);

    cprintf("x = %d, error = %f, driveL = %f\n",x,error,driveL);
    
    width = pixy.ccc.blocks[0].m_width;
    dist = getDistanceFromObject(width);
    cprintf("width = %d (pel) dist = %f (mm)\n",width,dist); 

 } else {
   // No block so stop the servos
   driveServos(0.0,0.0);
 }

 //delay(500);
}

float getDistanceFromObject(uint16_t _width) {
  float dist;
  // Change these parameters for your calibrated camera
  dist = 15276.2/((float)_width) - 38.3;
  return dist;
}

void driveServos(float _driveL, float _driveR) {
  servoL.writeMicroseconds(1500 + (int)_driveL);
  servoR.writeMicroseconds(1500 - (int)_driveR);
}
